import json
import imp
import os
import collections

import six
import yaml

__author__ = 'Nick Allen <nick.allen.cse@gmail.com>'


class Configuration(object):
	"""Base Configuration class"""

	file_extensions = ()

	def __init__(self, config_source):
		"""Expects config_source to be a dict"""
		self._source = config_source
		self.__config = None
		self.__raw_config = None

	def __getitem__(self, item):
		"""Lazy-load config using parse_source(), and act like the underlying dict"""
		return self.config[item]

	def get(self, key, default=None):
		"""Pass-through for dict.get()"""
		return self.config.get(key, default)

	def keys(self):
		"""Pass-through for dict.keys()"""
		return self.config.keys()

	def items(self):
		"""Pass-through for dict.items()"""
		return self.config.items()

	def copy(self):
		"""Pass-through for dict.copy()"""
		return self.config.copy()

	def values(self):
		"""Pass-through for dict.values"""
		return self.config.values()

	@property
	def config(self):
		"""Return dict config parsed for environment variables"""
		if self.__config is None:
			self.__config = self._replace_env(self.raw_config)

		return self.__config.copy()

	@property
	def raw_config(self):
		"""Returns raw dict config"""
		if self.__raw_config is None:
			self.__raw_config = self._filter_config(self.parse_source())

		return self.__raw_config.copy()

	@staticmethod
	def _replace_env(config_dict):
		"""Use os.path.expandvars to parse all values"""
		d = {}
		for key, value in config_dict.items():
			if isinstance(value, collections.Mapping):
				d[key] = Configuration._replace_env(value)
			elif isinstance(value, six.string_types):
				d[key] = os.path.expandvars(value)
			else:
				d[key] = value
		return d

	@staticmethod
	def _filter_config(config_dict):
		"""Returns keys in iterable that are uppercase and do not start with underscore"""
		return {key: val for key, val in config_dict.items() if key.isupper() and not key.startswith('_')}

	def parse_source(self):
		"""Default no-op returning self._source"""
		return self._source


class ModuleConfiguration(Configuration):
	"""Configuration from an imported python module, dot-string, or full file path to a module"""

	file_extensions = ('.py',)

	def parse_source(self):
		"""Load module if provided a dot-string, then parse config from module"""
		if isinstance(self._source, six.string_types):
			if os.path.isfile(self._source):
				module = imp.load_source(self._source, self._source)
			else:
				module = __import__(self._source, fromlist=['*'])
		else:
			module = self._source

		return {key: getattr(module, key) for key in dir(module)}


class JSONConfiguration(Configuration):
	"""Parse JSON file"""

	file_extensions = ('.json',)

	def parse_source(self):
		"""Read file and parse as JSON"""
		with open(self._source) as f:
			return json.load(f)


class YAMLConfiguration(Configuration):
	"""Parse YAML file"""

	file_extensions = ('.yml', '.yaml')

	def parse_source(self):
		"""Read file and parse as YAML"""
		with open(self._source) as f:
			return yaml.load(f)



supported_extensions = {}

for cls in Configuration.__subclasses__():
	for ext in cls.file_extensions:
		supported_extensions[ext] = cls


def parse_directory(path, name):
	"""
	Parse path for all supported config file types beginning with name

	Merges multiple configs with the same name into a single Configuration instance, overriding duplicate attributes in
	order files were loaded

	Ignores files with unsupported extensions
	"""
	configs = {}

	if os.path.isdir(path):
		for filename in os.listdir(path):
			if filename.startswith(name):
				ext = filename.split(name, 1)[1]
				if ext in supported_extensions:
					config = supported_extensions[ext](os.path.join(path, filename))
					configs.update(config.raw_config)

		return Configuration(configs)

	else:
		raise ValueError('`{}` is not a directory'.format(path))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

from marten import __version__, __doc__

def parse_requirements(requirements):
	with open(requirements) as f:
		return [l.strip('\n') for l in f if l.strip('\n') and not l.startswith('#')]

try:
	from StringIO import StringIO
except ImportError:
	from io import StringIO

import sys
stderr = sys.stderr
sys.stderr = StringIO()

try:
	from pypandoc import convert
	long_description = convert('README.md', 'rst')
except (ImportError, OSError):
	long_description = __doc__

sys.stderr = stderr

test_requirements = parse_requirements('test-requirements.txt')

setup(
	name='marten',
	version=__version__,
	packages=find_packages(exclude=('tests.*', 'tests',)),
	url='https://github.com/nick-allen/marten',
	license='MIT',
	author='Nick Allen',
	author_email='nick.allen.cse@gmail.com',
	description=__doc__,
	long_description=long_description,
	include_package_data=True,
	zip_safe=False,
	install_requires=parse_requirements('requirements.txt'),
	extras_require={
		'test': test_requirements
	},
	entry_points={
		'console_scripts': [
			'marten = marten.cli:marten_cli'
		]
	},
	test_suite='nose.collector',
	tests_require=test_requirements
)



"""Stupid simple Python configuration management"""

from __future__ import absolute_import

import os as _os

__version__ = '0.5.2'



# Attempt to auto-load a default configuration from files in <cwd>/.marten/ based on the MARTEN_ENV env variable
# MARTEN_ENV defaults to 'default'
_marten_dir = _os.path.join(_os.getcwd(), '.marten')
_os.environ.setdefault('MARTEN_ENV', 'default')

try:
	if _os.path.isdir(_marten_dir):
		from .configurations import parse_directory as _parse_directory
		config = _parse_directory(_marten_dir, _os.environ['MARTEN_ENV'])
	else:
		from .configurations import Configuration as _Configuration
		config = _Configuration({})
except ImportError:
	config = None



"""Console Entrypoints"""

from __future__ import print_function

import marten
import json

__author__ = 'Nick Allen <nick.allen.cse@gmail.com>'



def marten_cli():
	"""Outputs parsed config at martin.config as JSON"""
	if marten.config is not None:
		print(json.dumps(
			marten.config.config,
			sort_keys=True,
			indent=4
		))